package RalliJava;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.PrintStream;
import java.util.Scanner;
//import fi.jyu.mit.ohj2.Mjonot;

//import java.util.ArrayList;
//import java.util.List;

/**
 * @author idamk
 * @version 12.2.2025
 *
 */
public class Henkilot {
    
    private static final int MAX_HENKILOT = 5;

    int lkm = 0;
    private Henkilo[] alkiot;

    
    /**
     *  luodaan alustava taulukko
     */
    public Henkilot() {
        this.alkiot = new Henkilo[MAX_HENKILOT];
    }
    
    
    /**
     * @param henkilo lisättävä henkilö
     * @throws SailoException säilö
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Henkilot henkilot = new Henkilot();
     * Henkilo henkilo1 = new Henkilo(), henkilo2 = new Henkilo();
     * henkilot.getLkm() === 0;
     * henkilot.lisaa(henkilo1); henkilot.getLkm()===1;
     * henkilot.lisaa(henkilo2); henkilot.getLkm()===2;
     * henkilot.lisaa(henkilo1); henkilot.getLkm()===3;
     * henkilot.anna(0)===henkilo1;
     * henkilot.anna(1)===henkilo2;
     * henkilot.anna(2)===henkilo1;
     * henkilot.anna(3)===henkilo1; #THROWS IndexOutOfBoundsException
     * henkilot.lisaa(henkilo1); henkilot.getLkm()===4;
     * henkilot.lisaa(henkilo1); henkilot.getLkm()===5;
     * henkilot.lisaa(henkilo2); #THROWS SailoException
     * </pre>
     */
    public void lisaa(Henkilo henkilo) throws SailoException {
        if (lkm >= alkiot.length) throw new SailoException("Liikaa alkioita");
        this.alkiot[this.lkm] = henkilo;
        this.lkm++;
    }
    
    
    /**
     * Tallentaa henkilön tiedot tiedostoon
     * @param hakemisto mihin hakemistoon tallentaa
     * @throws SailoException jos tallennus epäonnistuu
     */
    public void tallenna(String hakemisto) throws SailoException  {
        File tiedosto = new File(hakemisto + "/nimet.dat");
        try (PrintStream out = new PrintStream(new FileOutputStream(tiedosto,true))){
            // jos asiat menee avauksessa hyvin, lähdetään käymään nimiä läpi    
            for (int i = 0; i < this.getLkm(); i++) {
                    Henkilo henkilo = this.anna(i);
                    out.println(henkilo.toString()); //pyydetään henkilöä muuttumaan merkkijonoksi
                }
            } catch (FileNotFoundException e) {
                throw new SailoException("Tiedosto " + tiedosto.getAbsolutePath());
            }
    }
    
    /**
     * @param hakemisto mistä haetaan
     * @throws SailoException jos tiedoston lukemisessa jotain ongelmaa
     */
    public void lueTiedostosta(String hakemisto) throws SailoException {
        String tiedostonNimi = hakemisto + "/nimet.dat";
        File tiedosto = new File(tiedostonNimi);
        
        try (Scanner in = new Scanner(new FileInputStream(tiedosto))) {
            while (in.hasNext() ) { // jos on seuraava olemassa niin luetaan seuraava nimi
                String s = in.nextLine();
                if (s == null || "".equals(s) || s.charAt(0) == ';') continue;
                Henkilo henkilo = new Henkilo();
                henkilo.parse(s); //TODO: PARSE MJONOT.JYU juttu!!!!!
                
                lisaa(henkilo);
            }
        } catch (FileNotFoundException e) {
            throw new SailoException("Ei saa luettua tiedostoa " + tiedostonNimi);
        }
        
    }


    /**
     * @param rallinTunnusNro tarkasteltavan rallin tunnusnumero
     * @return listan ilmottautuneista
     */
    //public List<Henkilo> annaOsallistujat(int rallinTunnusNro) {
    //    List<Henkilo> ilmottautuneet = new ArrayList<Henkilo>();
    //    return null;
    //}
    
    /**
     * Haetaan kaikki henkilön osallistumat rallit. 
     * Käy rallit läpi ja etsii sieltä ne, 
     * mihin kysytty henkilö on osallistunut
     * @param tunnusNro rallin tunnusnumero
     * @return lista ralleista johon haluttu henkilö on osallistunut
     */
    /*public List<Ralli> annaRallit(int tunnusNro) {
        List<Ralli> loydetyt = new ArrayList<Ralli>();
        // Käy rallit läpi ja etsii sieltä ne, mihin kysytty henkilö on osallistunut
        for (Ralli ralli:alkiot) //ITERAATTORI
            if(ralli.getHenkilonTunnusNro() == tunnusNro) loydetyt.add(ralli);
        return loydetyt;
    
*/

    /**
     * @return lkm
     */
    public int getLkm() {
        return lkm;
    }
    
    
    /**
     * @param i indeksi
     * @return henkilön
     * @throws IndexOutOfBoundsException laiton indeksi
     */
    public Henkilo anna(int i) throws IndexOutOfBoundsException {
        if (i < 0 || this.lkm <= i)
                throw new IndexOutOfBoundsException("Laiton indeksi senkin kaulin: " + i);
        return alkiot[i];
    }
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Henkilot henkilot = new Henkilot();
        
        try {
            henkilot.lueTiedostosta("kerho");
        } catch (SailoException ex) {
            System.err.println(ex.getMessage());
        }
        
        
        Henkilo rovis = new Henkilo();
        Henkilo asunmaa = new Henkilo();
        Henkilo elfyn = new Henkilo();
        
        rovis.rekisteroi();
        rovis.vastaahlo();
        asunmaa.rekisteroi();
        asunmaa.vastaahlo();
        elfyn.rekisteroi();
        elfyn.vastaahlo();
        
        try {
            henkilot.lisaa(rovis);
            henkilot.lisaa(asunmaa);
            henkilot.lisaa(elfyn); 
            
            System.out.println("==================== Testi ======================");
            
            for (int i = 0; i < henkilot.getLkm(); i++) {
                Henkilo henkilo = henkilot.anna(i);
                System.out.println("Henkilo nro: " + (i+1));
                henkilo.tulosta(System.out);
            }
            
        } catch (SailoException e) {
                System.err.println("Liikaa alkioita :0"); //kun alkioita enemmän kun MAX sallii
        }
        
        // tallentaminen hakemistoon "kerho"
        try {
            henkilot.tallenna("kerho");
        } catch (SailoException e) {
            System.err.println(e.getMessage());
        }
        

    }



}
